package main

import (
	"embed"
	"errors"
	"fmt"
	"io"
	"io/fs"
	"net/url"
	"os"
	"path/filepath"
	"text/template"

	"github.com/kelseyhightower/envconfig"
)

type Options struct {
	DSN           string   `envconfig:"DSN"`
	ConfigURL     string   `envconfig:"CONFIG_URL"`     // URL called for configuration, see https://developer.signalwire.com/freeswitch/FreeSWITCH-Explained/Modules/mod_xml_curl_1049001/ (dialplan|directory|phrases will be fetched from to this URL)
	CaptureServer string   `envconfig:"CAPTURE_SERVER"` // if set, this server gets a copy of all SIP messages. Example: udp:homer.domain.com:5060;hep=3;capture_id=100
	LogLevel      string   `envconfig:"LOG_LEVEL" default:"info"`
	Gateway       Gateway  `envconfig:"GATEWAY"`
	Modules       []string `envconfig:"MODULES" default:"mod_event_socket,mod_sofia,mod_db,mod_dialplan_xml,mod_g723_1,mod_g729,mod_amr,mod_b64,mod_opus,mod_av,mod_sndfile,mod_native_file,mod_png,mod_local_stream,mod_tone_stream,mod_lua,mod_say_en"`
	StaticXMLDir  string   `envconfig:"STATIC_XML_DIR" default:"/config"`
}

type Gateway struct {
	Name     string
	Username string
	Password string
	Extra    map[string]string
}

var (
	//go:embed autoload_configs
	autoloadConfigsFS        embed.FS
	autoloadConfigsTemplates = template.Must(template.New("autoload_configs").ParseFS(autoloadConfigsFS, "autoload_configs/*"))
	options                  Options
	vars                     = map[string]string{ // this is used to fill in what is normally in vars.xml
		"sound_prefix":                 "$${sounds_dir}/en/us/callie",
		"domain":                       "$${local_ip_v4}",
		"domain_name":                  "$${domain}",
		"hold_music":                   "local_stream://moh",
		"use_profile":                  "external",
		"rtp_sdes_suites":              "AEAD_AES_256_GCM_8|AEAD_AES_128_GCM_8|AES_CM_256_HMAC_SHA1_80|AES_CM_192_HMAC_SHA1_80|AES_CM_128_HMAC_SHA1_80|AES_CM_256_HMAC_SHA1_32|AES_CM_192_HMAC_SHA1_32|AES_CM_128_HMAC_SHA1_32|AES_CM_128_NULL_AUTH",
		"global_codec_prefs":           "OPUS,G722,PCMU,PCMA,H264,VP8",
		"outbound_codec_prefs":         "OPUS,G722,PCMU,PCMA,H264,VP8",
		"xmpp_client_profile":          "xmppc",
		"xmpp_server_profile":          "xmpps",
		"bind_server_ip":               "auto",
		"unroll_loops":                 "true",
		"outbound_caller_name":         "FreeSWITCH",
		"outbound_caller_id":           "0000000000",
		"call_debug":                   "false",
		"console_loglevel":             "info",
		"default_areacode":             "918",
		"default_country":              "US",
		"presence_privacy":             "false",
		"au-ring":                      "%(400,200,383,417);%(400,2000,383,417)",
		"be-ring":                      "%(1000,3000,425)",
		"ca-ring":                      "%(2000,4000,440,480)",
		"cn-ring":                      "%(1000,4000,450)",
		"cy-ring":                      "%(1500,3000,425)",
		"cz-ring":                      "%(1000,4000,425)",
		"de-ring":                      "%(1000,4000,425)",
		"dk-ring":                      "%(1000,4000,425)",
		"dz-ring":                      "%(1500,3500,425)",
		"eg-ring":                      "%(2000,1000,475,375)",
		"es-ring":                      "%(1500,3000,425)",
		"fi-ring":                      "%(1000,4000,425)",
		"fr-ring":                      "%(1500,3500,440)",
		"hk-ring":                      "%(400,200,440,480);%(400,3000,440,480)",
		"hu-ring":                      "%(1250,3750,425)",
		"il-ring":                      "%(1000,3000,400)",
		"in-ring":                      "%(400,200,425,375);%(400,2000,425,375)",
		"jp-ring":                      "%(1000,2000,420,380)",
		"ko-ring":                      "%(1000,2000,440,480)",
		"pk-ring":                      "%(1000,2000,400)",
		"pl-ring":                      "%(1000,4000,425)",
		"ro-ring":                      "%(1850,4150,475,425)",
		"rs-ring":                      "%(1000,4000,425)",
		"ru-ring":                      "%(800,3200,425)",
		"sa-ring":                      "%(1200,4600,425)",
		"tr-ring":                      "%(2000,4000,450)",
		"uk-ring":                      "%(400,200,400,450);%(400,2000,400,450)",
		"us-ring":                      "%(2000,4000,440,480)",
		"bong-ring":                    "v",
		"beep":                         "%(1000,0,640)",
		"sit":                          "%(274,0,913.8);%(274,0,1370.6);%(380,0,1776.7)",
		"df_us_ssn":                    "(?!219099999|078051120)(?!666|000|9d{2})d{3}(?!00)d{2}(?!0{4})d{4}",
		"df_luhn":                      "?:4[0-9]{12}(?:[0-9]{3})?|5[1-5][0-9]{14}|3[47][0-9]{13}|3(?:0[0-5]|[68][0-9])[0-9]{11}|6(?:011|5[0-9]{2})[0-9]{12}|(?:2131|1800|35d{3})d{11}",
		"default_provider":             "example.com",
		"default_provider_username":    "joeuser",
		"default_provider_password":    "password",
		"default_provider_from_domain": "example.com",
		"default_provider_register":    "false",
		"default_provider_contact":     "5000",
		"sip_tls_version":              "tlsv1,tlsv1.1,tlsv1.2",
		"sip_tls_ciphers":              "ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH",
		"internal_auth_calls":          "true",
		"internal_sip_port":            "5060",
		"internal_tls_port":            "5061",
		"internal_ssl_enable":          "false",
		"external_auth_calls":          "false",
		"external_sip_port":            "5080",
		"external_tls_port":            "5081",
		"external_ssl_enable":          "false",
		"rtp_video_max_bandwidth_in":   "3mb",
		"rtp_video_max_bandwidth_out":  "3mb",
		"suppress_cng":                 "true",
		"rtp_liberal_dtmf":             "true",
		"video_mute_png":               "$${images_dir}/default-mute.png",
		"video_no_avatar_png":          "$${images_dir}/default-avatar.png",
	}
)

func main() {
	if err := envconfig.Process("", &options); err != nil {
		panic(err)
	}

	if options.ConfigURL != "" {
		options.Modules = append(options.Modules, "mod_xml_curl")
	}

	// autoload_configs
	for k, v := range vars {
		fmt.Printf("<X-PRE-PROCESS cmd=\"set\" data=\"%s=%s\"/>\n", k, v)
	}

	fmt.Println("<section name=\"configuration\" description=\"Various Configuration\">")

	configs, err := autoloadConfigsFS.ReadDir("autoload_configs")
	if err != nil {
		panic(err)
	}

	for _, file := range configs {
		if file.IsDir() {
			fmt.Println("skipping dir: ", file.Name())
			continue
		}
		if err := autoloadConfigsTemplates.ExecuteTemplate(os.Stdout, file.Name(), options); err != nil {
			panic(err)
		}
	}

	fmt.Println("</section>")

	staticConfigDirectories, err := os.ReadDir(options.StaticXMLDir)
	if err != nil {
		if errors.Is(err, os.ErrNotExist) {
			return
		}
		panic(err)
	}

	for _, entry := range staticConfigDirectories {
		if !entry.IsDir() {
			continue
		}
		fmt.Printf("<section name=\"%s\">\n", entry.Name())

		filepath.WalkDir(filepath.Join(options.StaticXMLDir, entry.Name()), func(path string, d fs.DirEntry, err error) error {
			if d.IsDir() {
				return nil
			}

			fmt.Printf("<!-- %s -->\n", path)
			if err != nil {
				fmt.Printf("<!-- ERROR: %v -->", err)
				return err
			}

			f, err := os.Open(path)
			if err != nil {
				panic(err)
			}
			defer f.Close()

			data, err := io.ReadAll(f)
			if err != nil {
				panic(err)
			}
			fmt.Println(string(data))

			return nil
		})
		fmt.Println("</section>")
	}
}

func (g *Gateway) Decode(value string) error {
	u, err := url.Parse(value)
	if err != nil {
		return err
	}

	g.Name = u.Hostname()
	g.Username = u.User.Username()
	if password, ok := u.User.Password(); ok {
		g.Password = password
	}

	g.Extra = make(map[string]string)
	for k, v := range u.Query() {
		if len(v) > 0 {
			g.Extra[k] = v[0]
		}
	}
	return nil
}
